@@ -1,12 +1,14 @@
 """
 Lowest level connection
 """
+import logging
+
 import six
 from botocore.session import get_session
-import logging
+
 from .util import pythonic
-from .exceptions import TableError, QueryError, PutError, DeleteError, UpdateError, GetError, ScanError
 from ..types import HASH, RANGE
+from pynamodb.exceptions import TableError, QueryError, PutError, DeleteError, UpdateError, GetError, ScanError
 from pynamodb.constants import (
     RETURN_CONSUMED_CAPACITY_VALUES, RETURN_ITEM_COLL_METRICS_VALUES, COMPARISON_OPERATOR_VALUES,
     RETURN_ITEM_COLL_METRICS, RETURN_CONSUMED_CAPACITY, RETURN_VALUES_VALUES, ATTR_UPDATE_ACTIONS,
@@ -22,6 +24,7 @@ from pynamodb.constants import (
     CONSUMED_CAPACITY, CAPACITY_UNITS
     )
 
+
 log = logging.getLogger(__name__)
 log.addHandler(logging.NullHandler())
 
@@ -211,7 +214,7 @@ class Connection(object):
 
     def dispatch(self, operation_name, operation_kwargs):
         """
-        Dispatches `operation_name` with arguments ``operation_kwargs`
+        Dispatches `operation_name` with arguments `operation_kwargs`
         """
         if operation_name not in [DESCRIBE_TABLE, LIST_TABLES, UPDATE_TABLE, DELETE_TABLE, CREATE_TABLE]:
             if pythonic(RETURN_CONSUMED_CAPACITY) not in operation_kwargs:
@@ -257,12 +260,12 @@ class Connection(object):
         """
         Returns an endpoint connection to `self.region`
         """
-        if self._endpoint is None:
-            if self.host:
-                self._endpoint = self.service.get_endpoint(self.region, endpoint_url=self.host)
-            else:
-                self._endpoint = self.service.get_endpoint(self.region)
-        return self._endpoint
+        end_point = None
+        if self.host:
+            end_point = self.service.get_endpoint(self.region, endpoint_url=self.host)
+        else:
+            end_point = self.service.get_endpoint(self.region)
+        return end_point
 
     def get_meta_table(self, table_name, refresh=False):
         """
