@@ -2,10 +2,14 @@
 Test model API
 """
 import copy
+from datetime import datetime
+from unittest import TestCase
+
 import six
+
 from pynamodb.throttle import Throttle
 from pynamodb.connection.util import pythonic
-from pynamodb.connection.exceptions import TableError
+from pynamodb.exceptions import TableError
 from pynamodb.types import RANGE
 from pynamodb.constants import (
     ITEM, STRING_SHORT, ALL, KEYS_ONLY, INCLUDE, REQUEST_ITEMS, UNPROCESSED_KEYS,
@@ -17,13 +21,15 @@ from pynamodb.indexes import (
     IncludeProjection, KeysOnlyProjection, Index
 )
 from pynamodb.attributes import (
-    UnicodeAttribute, NumberAttribute, BinaryAttribute,
+    UnicodeAttribute, NumberAttribute, BinaryAttribute, UTCDateTimeAttribute,
     UnicodeSetAttribute, NumberSetAttribute, BinarySetAttribute)
-from unittest import TestCase
 from .response import HttpOK, HttpBadRequest
 from .data import (
     MODEL_TABLE_DATA, GET_MODEL_ITEM_DATA, SIMPLE_MODEL_TABLE_DATA,
-    BATCH_GET_ITEMS, SIMPLE_BATCH_GET_ITEMS)
+    BATCH_GET_ITEMS, SIMPLE_BATCH_GET_ITEMS, COMPLEX_TABLE_DATA,
+    COMPLEX_ITEM_DATA
+)
+
 
 # Py2/3
 if six.PY3:
@@ -117,6 +123,15 @@ class UserModel(Model):
     callable_field = NumberAttribute(default=lambda: 42)
 
 
+class ComplexKeyModel(Model):
+    """
+    This model has a key that must be serialized/deserialized properly
+    """
+    table_name = 'ComplexKey'
+    name = UnicodeAttribute(hash_key=True)
+    date_created = UTCDateTimeAttribute(default=datetime.utcnow)
+
+
 class ModelTestCase(TestCase):
     """
     Tests for the models API
@@ -228,6 +243,10 @@ class ModelTestCase(TestCase):
             req.return_value = HttpOK(), MODEL_TABLE_DATA
             item = UserModel('foo', 'bar')
 
+        with patch(PATCH_METHOD) as req:
+            req.return_value = HttpOK(), {}
+            self.assertRaises(item.DoesNotExist, item.refresh)
+
         with patch(PATCH_METHOD) as req:
             req.return_value = HttpOK(GET_MODEL_ITEM_DATA), GET_MODEL_ITEM_DATA
             item.refresh()
@@ -235,6 +254,18 @@ class ModelTestCase(TestCase):
                 item.user_name,
                 GET_MODEL_ITEM_DATA.get(ITEM).get('user_name').get(STRING_SHORT))
 
+    def test_complex_key(self):
+        """
+        Model with complex key
+        """
+        with patch(PATCH_METHOD) as req:
+            req.return_value = HttpOK(), COMPLEX_TABLE_DATA
+            item = ComplexKeyModel('test')
+
+        with patch(PATCH_METHOD) as req:
+            req.return_value = HttpOK(COMPLEX_ITEM_DATA), COMPLEX_ITEM_DATA
+            item.refresh()
+
     def test_delete(self):
         """
         Model.delete
@@ -491,7 +522,7 @@ class ModelTestCase(TestCase):
 
         with patch(PATCH_METHOD) as req:
             req.return_value = HttpOK({}), {}
-            self.assertIsNone(UserModel.get('foo', 'bar'))
+            self.assertRaises(UserModel.DoesNotExist, UserModel.get, 'foo', 'bar')
 
     def test_batch_get(self):
         """
@@ -506,6 +537,26 @@ class ModelTestCase(TestCase):
             item_keys = ['hash-{0}'.format(x) for x in range(10)]
             for item in SimpleUserModel.batch_get(item_keys):
                 self.assertIsNotNone(item)
+            params = {
+                'return_consumed_capacity': 'TOTAL',
+                'request_items': {
+                    'SimpleModel': {
+                        'Keys': [
+                            {'user_name': {'S': 'hash-9'}},
+                            {'user_name': {'S': 'hash-8'}},
+                            {'user_name': {'S': 'hash-7'}},
+                            {'user_name': {'S': 'hash-6'}},
+                            {'user_name': {'S': 'hash-5'}},
+                            {'user_name': {'S': 'hash-4'}},
+                            {'user_name': {'S': 'hash-3'}},
+                            {'user_name': {'S': 'hash-2'}},
+                            {'user_name': {'S': 'hash-1'}},
+                            {'user_name': {'S': 'hash-0'}}
+                        ]
+                    }
+                }
+            }
+            self.assertEqual(params, req.call_args[1])
 
         with patch(PATCH_METHOD) as req:
             req.return_value = HttpOK(), MODEL_TABLE_DATA
